clc; clear;

% Inline function for the Black Scholes call.
BSC = inline('s.*normcdf((log(s/K) + (r+v^2/2)*T)/v/sqrt(T)) - K*exp(-r*T)*normcdf((log(s/K) + (r+v^2/2)*T)/v/sqrt(T) - v*sqrt(T))',...
	's','K','r','v','T');

% Inline function for the Black Scholes put.
BSP = inline('K*exp(-r*T)*normcdf(-(log(s/K) + (r+v^2/2)*T)/v/sqrt(T) + v*sqrt(T)) - s*normcdf(-(log(s/K) + (r+v^2/2)*T)/v/sqrt(T))',...
	's','K','r','v','T');

% Input the price levels and dates.
[P dates] = xlsread('SP 100 Prices.xls', 'Sheet1', 'A2:C1003');
Prices = P(:,2);

% Risk free rate
rf = 0;

% Starting values for optimization.
% Use estimates from Duan's paper as starting values.
start  = [0.000015 0.19 0.72 0.007];
params = fminsearch(@(b) findGARCH_LL(b,Prices,rf), start)
alpha0 = params(1);
alpha1 = params(2);
beta1  = params(3);
lambda = params(4);

% Find the standard deviation (sigma) implied by the parameters.
% Assume 360 days per year.
variance = alpha0 / (1 - alpha1*(1+lambda^2) - beta1);
sigma = sqrt(variance)*sqrt(360);

% Option features.
N = 180;      % Maturity in days.
T = N/360;    % Maturity in years.
Spot = 100;   % Spot price.
K = 90;       % Strike price.

% Number of simulations and stock price initialization.
Nsims = 5000;
S = zeros(N,Nsims);
S(1,:) = Spot;
e(1) = 0;
h(1) = variance;

% Simulate paths for the stock price, and retain the terminal prices S(T).
for i=1:Nsims;
	for t=2:N
		h(t) = alpha0 + alpha1*(e(t-1) - lambda*sqrt(h(t-1)))^2 + beta1*h(t-1);
		e(t) = sqrt(h(t))*randn(1);
		S(t,i) = S(t-1,i)*exp(rf - 0.5*h(t) + e(t));
	end
	ST(i) = S(end,i);
end

% Plot the stock price paths.
plot(S)

% Calculate the GARCH and Black-Scholes calls and puts.
GARCHCall = exp(-rf*T)*mean(max(ST-K,0)) 
BSCall = BSC(Spot,K,rf,sigma,T)

GARCHPut  = exp(-rf*T)*mean(max(K-ST,0))
BSPut  = BSP(Spot,K,rf,sigma,T)

