function y = findGARCH_LL(params,S,rf);

% Finds log-likelihood for the GARCH option pricing model.
alpha0 = params(1);
alpha1 = params(2);
beta1  = params(3);
lambda = params(4);

N = length(S);

% Define the returns.
r = log(S(2:N)./S(1:N-1));
r = [0; r];

% Initialize the GARCH log-likelihood at t=1.
h(1) = var(r);
e(1) = 0;
LL(1) = -0.5*e(1)^2/h(1) - 0.5*log(2*pi*h(1));

% Find the rest of the GARCH log-likelihood.
for t=2:N
    h(t) = alpha0 + alpha1*(e(t-1) - lambda*sqrt(h(t-1)))^2 + beta1*h(t-1);
	e(t) = log(S(t)/S(t-1)) - rf + 0.5*h(t);
    LL(t) = -0.5*e(t)^2/h(t) - 0.5*log(2*pi*h(t));
end

% Return the negative log-likelihood.
y = -sum(LL);

